// Tennisball1.java
// Friction proportional to v^2

package ch.aplu.tut;

import ch.aplu.android.*;
import android.graphics.Point;

public class TennisBall1 extends Actor
{
  private final GGVector g = new GGVector(0, 9.81);
  private final double m = 0.057; // kg
  private final double R = 0.005; // Air friction (N per (m/s)^2)
  private double dimFactor;
  private GGVector r;
  private GGVector v;
  private double dt;  // in s

  public TennisBall1(GGVector v, double dimFactor)
  {
    super("tennisball");
    // Initial condition
    this.v = v;
    this.dimFactor = dimFactor;
  }
  
  public void reset()
  {
    // Initial condition
    r = new GGVector(getX() / dimFactor, getY() / dimFactor);
    dt = gameGrid.getSimulationPeriod() / 1000.0;  // in s
  }
  
  public void act()
  {
    GGVector f = g.mult(m).add(v.mult(-R * v.magnitude()));
    GGVector a = f.mult(1 / m);
    GGVector vnew = v.add(a.mult(dt));
    GGVector rnew = r.add(v.mult(dt));
    setLocation(new Location(toPix(rnew).x, toPix(rnew).y));
    gameGrid.getBg().drawLine(toPix(r), toPix(rnew));
    r = rnew;
    v = vnew;
    
    if (!isInGrid())
      removeSelf();
  }
  
  private Point toPix(GGVector z)
  {
    return new Point((int)(dimFactor * z.x + 0.5), 
      (int)(dimFactor * z.y + 0.5));
  }
}
