// SearchDevice.java
// Searching for a particular Bluetooth device
// either as paired device (coupled) or with device discovery

package ch.aplu.btex;

import ch.aplu.android.*;
import ch.aplu.android.bluetooth.*;
import android.graphics.Color;
import android.bluetooth.*;

public class SearchDevice extends GameGrid
{
  private GGConsole c;

  public void main()
  {
    c = GGConsole.init();
    String btName = askName();
    searchDevice(btName);
  }

  private String askName()
  {
    GGPreferences prefs = new GGPreferences(this);
    String oldName = prefs.retrieveString("BluetoothName");
    String newName = null;
    while (newName == null)
    {
      newName = GGInputDialog.show(c.getActivity(), "Search Device", "Enter Bluetooth Name",
        oldName == null ? "" : oldName);
    }
    prefs.storeString("BluetoothName", newName);
    return newName;
  }

  /*
   * Searches for device, first in set of paired devices than
   * by a Bluetooth discovery. Returns null, if device not found.
   */
  private BluetoothDevice searchDevice(String btName)
  {
    BluetoothAdapter adapter =
      BluetoothDiscovery.getBluetoothAdapter(this);
    if (adapter == null)
    {
      c.println("Bluetooth not supported");  // e.g. in emulator
      return null;
    }
    c.println("Testing if Bluetooth is enabled...");
    if (!adapter.isEnabled())
    {
      requestBluetoothEnable();
      if (adapter.isEnabled())
        c.println("Enabled.");
      else
      {
        c.println("Not enabled.");
        return null;
      }
    }
    c.println("Searching for paired device '" + btName + "'...");
    BluetoothDiscovery bd = new BluetoothDiscovery(this);
    BluetoothDevice device = bd.getPairedDevice(btName);
    if (device != null)
    {
      c.println("Paired device '" + btName + "' found.");
      return device;
    }

    c.println("Paired device '" + btName + "' not found.");
    c.println("Perform device discovery for device '" + btName + "'...");
    device = bd.discoverBluetoothDevice(btName);
    // Blocks until device is discovered or discovery is finished
    if (device == null)
    {
      c.println("Device '" + btName + "' not found.");
      c.println("(Not in reach or not visible)");
      return null;
    }

    c.println("Device " + btName + " found.");
    return device;
  }

}
