// BtServerApp.java
// Data exchange with BtClientApp or BtClientSE

package ch.aplu.btserver;

import ch.aplu.android.*;
import ch.aplu.android.bluetooth.*;
import android.bluetooth.*;
import java.io.*;
import ch.aplu.util.Monitor;

public class BtServerApp extends GameGrid
  implements BluetoothServerListener, GGNavigationListener
{
  private class ReceiverThread extends Thread
  {
    private boolean isRunning;

    public void run()
    {
      isRunning = true;
      while (isRunning)
      {
        try
        {
          int data = dis.readInt();
          c.println("Got data: " + data);
          if (data == disconnectTag)
          {
            c.println("Got disconnect notification");
            isRunning = false;
            continue;
          }
          // Echo inverse
          dos.writeInt(-data);
          dos.flush();
        }
        catch (IOException ex)
        {
          c.println("IOExeption in blocking readInt()");
          isRunning = false;
        }
      }
      c.println("Receiver thread terminated");
      Monitor.wakeUp(); // Wake-up sleeping notifyConnection()
    }

  }

  //
  private final String serviceName = "BtServer";
  private final int disconnectTag = 9999;
  private BluetoothAdapter adapter;
  private BluetoothServer bs;
  private DataInputStream dis;
  private DataOutputStream dos;
  private int nbConnections = 0;
  private boolean isClientConnected = false;
  private GGConsole c;

  public void main()
  {
    c = GGConsole.init();
    addNavigationListener(this);

    adapter = BluetoothDiscovery.getBluetoothAdapter(this);
    c.println("Creating server with Bluetooth name '"
      + adapter.getName() + "'");
    bs = new BluetoothServer(adapter, serviceName, this);
    c.println("Waiting for a Bluetooth client to connect ");
    c.println("exposing service '" + serviceName + "'");
  }

  public boolean notifyClientConnection(BluetoothDevice device,
    InputStream is, OutputStream os)
  {
    c.println("notifyClientConnection() starting");
    nbConnections++;
    c.println("Client '" + device.getName() + "' connected as # " + nbConnections);
    isClientConnected = true;
    c.println("Retrieving input and output streams");
    dis = new DataInputStream(is);
    dos = new DataOutputStream(os);
    new ReceiverThread().start();
    c.println("Starting receiver thread");
    c.println("notifyClientConnection() blocking now");
    Monitor.putSleep();  // Thread goes to sleep
    c.println("notifyClientConnection() wakes up");
    c.println("Lost client connection. Server will close streams and socket");
    c.println("Waiting for next client...");
    isClientConnected = false;
    return true;
  }

  // Is called by the system
  public void onPause()
  {
    if (isClientConnected)
      sendDisconnectTag();
    if (bs != null)
      bs.abort();
    super.onPause();
  }

  private void sendDisconnectTag()
  {
    try
    {
      dos.writeInt(disconnectTag); 
      dos.flush();
    }
    catch (IOException ex)
    {
    }
  }

  public void navigationEvent(GGNavigationEvent event)
  {
    showToast("Not implemented");
  }

}
