// CardEx07.java
// ToolBar

package ch.aplu.cardex07;

import android.graphics.Color;
import ch.aplu.jcardgame.*;
import ch.aplu.android.*;

public class CardEx07 extends CardGame
{
  public enum Suit
  {
    SPADES, HEARTS, DIAMONDS, CLUBS
  }

  public enum Rank
  {
    ACE, KING, QUEEN, JACK, TEN, NINE, EIGHT, SEVEN, SIX, FIVE, FOUR, THREE, TWO
  }
  //

  private int nbRanks = Rank.values().length;
  private Deck deck;
  private Hand hand;
  private ToolBarStack spades;
  private ToolBarStack hearts;
  private ToolBarStack diamonds;
  private ToolBarStack clubs;
  private ToolBarSeparator separator0;
  private ToolBarSeparator separator1;
  private ToolBarSeparator separator2;
  private ToolBarSeparator separator3;
  private ToolBarItem okBtn;
  private GGPushButton sortBtn;

  public CardEx07()
  {
    super(Color.rgb(20, 80, 0), Color.WHITE, BoardType.HORZ_SQUARE, windowZoom(600)); 
  }

  public void main()
  {
    deck = new Deck(Suit.values(), Rank.values(), "cover");
    hand = new Hand(deck);
    hand.setView(this, new RowLayout(new Location(300, 300), 290));
    hand.setTouchEnabled(true);
    hand.draw();
    initToolBar();
    sortBtn = new GGPushButton("sortbtn");
    sortBtn.addPushButtonListener(
      new GGPushButtonAdapter()
      {
        public void buttonPressed(GGPushButton btn)
        {
          hand.sort(Hand.SortType.RANKPRIORITY, true);
        }
      });
    addActor(sortBtn, new Location(300, 500).toReal());
  }

  private void initToolBar()
  {
    double currentZoomFactor = getZoomFactor();
    setZoomFactor(2);  // Don't scale down too much
    spades = new ToolBarStack("spades", nbRanks);
    int h = 60; // zoomFactor * item height
    separator0 = new ToolBarSeparator(10, h, Color.BLACK);
    hearts = new ToolBarStack("hearts", nbRanks);
    separator1 = new ToolBarSeparator(10, h, Color.BLACK);
    diamonds = new ToolBarStack("diamonds", nbRanks);
    separator2 = new ToolBarSeparator(10, h, Color.BLACK);
    clubs = new ToolBarStack("clubs", nbRanks);
    separator3 = new ToolBarSeparator(20, h, Color.BLACK);
    okBtn = new ToolBarItem("ok30", 2);
    setZoomFactor(currentZoomFactor);

    ToolBar toolBar = new ToolBar(this);
    toolBar.addItem(spades, separator0, hearts,
      separator1, diamonds, separator2, clubs, separator3, okBtn);
    toolBar.add(new Location(10, 10));

    toolBar.addToolBarListener(new ToolBarAdapter()
    {
      public void pressed(ToolBarItem item)
      {
        if (item == okBtn)
        {
          okBtn.show(1);
          Card card = getSelectedCard(deck);
          if (card != null)
          {
            if (!hand.insert(card, true))
              showToast("Failed to insert card " + card
                + "\n(no duplication allowed)");
          }
        }
        else
        {
          ToolBarStack stackItem = (ToolBarStack)item;
          if (stackItem.isSelected())
            stackItem.showNext();
          else
          {
            deselectAll();
            stackItem.setSelected(true);
          }
        }
      }

      public void released(ToolBarItem item)
      {
        if (item == okBtn)
          item.show(0);
      }

    });
  }

  private void deselectAll()
  {
    spades.setSelected(false);
    hearts.setSelected(false);
    diamonds.setSelected(false);
    clubs.setSelected(false);
  }

  private Card getSelectedCard(Deck deck)
  // Assumes that only one card stack is selected
  // Returns null, if no card is selected
  {
    Card card = null;
    int[] ids = ToolBarStack.getSelectedItemIds();
    for (int i = 0; i < ids.length; i++)
    {
      int id = ids[i];
      if (id != -1)  // Selected item found
      {
        deck.getSuit(i);
        deck.getRank(id);
        card = new Card(deck, deck.getSuit(i), deck.getRank(id));
        break;
      }
    }
    return card;
  }

}