// CardEx09.java
// Selecting random cards

package ch.aplu.cardex09;

import ch.aplu.jcardgame.*;
import ch.aplu.android.*;
import android.graphics.Color;

public class CardEx09 extends CardGame
{
  public enum Suit
  {
    SPADES, HEARTS, DIAMONDS, CLUBS
  }

  public enum Rank
  {
    ACE, KING, QUEEN, JACK, TEN, NINE, EIGHT, SEVEN, SIX
  }

  private Deck deck;
  private Hand upperHand;
  private Hand lowerHand;

  public CardEx09()
  {
    super(Color.rgb(20, 80, 0), Color.WHITE, BoardType.HORZ_SQUARE, windowZoom(600)); 
  }

  public void main()
  {
    deck = new Deck(Suit.values(), Rank.values(), "cover");
    Hand[] hands = deck.dealingOut(2, 5, true);
    upperHand = hands[0];
    upperHand.setView(this, new RowLayout(new Location(300, 100), 400));
    lowerHand = hands[1];
    lowerHand.setView(this, new RowLayout(new Location(300, 500), 400));
    sortAndDrawHands();
    initToolBar();
  }

  private void sortAndDrawHands()
  {
    upperHand.sort(Hand.SortType.RANKPRIORITY, true);
    lowerHand.sort(Hand.SortType.RANKPRIORITY, true);
  }

  private void initToolBar()
  {
    double currentZoomFactor = getZoomFactor();
    setZoomFactor(2);  // Don't scale down too much
    final ToolBarItem upBtn = new ToolBarItem("up30", 2);
    final ToolBarStack numbers = new ToolBarStack("number30", 10);
    final ToolBarItem downBtn = new ToolBarItem("down30", 2);
    setZoomFactor(currentZoomFactor);
    numbers.show(3);  // Default start number
    ToolBar toolBar = new ToolBar(this);
    toolBar.addItem(upBtn, numbers, downBtn);
    int width = toolBar.getWidth();
    Location center = new Location(300, 300).toReal();
    toolBar.add(new Location(center.x - width / 2, center.y - 30));  // Real coordinates

    toolBar.addToolBarListener(new ToolBarAdapter()
    {
      public void pressed(ToolBarItem item)
      {
        if (item == upBtn)
        {
          upBtn.show(1);
          Hand.randomBatchTransfer(numbers.getItemId(),
            lowerHand, upperHand, false);
          sortAndDrawHands();
        }
        if (item == downBtn)
        {
          downBtn.show(1);
          Hand.randomBatchTransfer(numbers.getItemId(),
            upperHand, lowerHand, false);
          sortAndDrawHands();
        }
        if (item == numbers)
          ((ToolBarStack)item).showNext();
      }

      public void released(ToolBarItem item)
      {
        if (item == upBtn)
          upBtn.show(0);
        if (item == downBtn)
          downBtn.show(0);
      }

    });
  }
}