// NxtAgent.java

package ch.aplu.nxtagent;

import ch.aplu.android.*;
import ch.aplu.android.nxt.*;
import ch.aplu.tcp.*;
import ch.aplu.util.Monitor;

public class NxtAgent extends GameGrid
  implements NxtConnectionListener, TcpNodeListener
{
  private final String VERSION = "1.0";
  private NxtRover rover;
  private TcpNode node;
  private final String nickname = "NxtAgent";
  private static String sessionID = "777";
  protected GGTextField[] tf = new GGTextField[8];
  private volatile boolean isRunning = true;

  public NxtAgent()
  {
    super(WHITE);
  }

  public void main()
  {
    getBg().clear(BLUE);
    for (int i = 0; i < tf.length; i++)
    {
      tf[i] = new GGTextField(new Location(5, 20 + 20 * i), true);
      tf[i].setFontSize(10);
      tf[i].show();
    }

    tf[0].setText("NxtAgent Version " + VERSION);
    tf[1].setText("Connecting Bluetooth...");

    rover = new NxtRover(this);
    rover.addNxtConnectionListener(this);
    if (rover.connect() != NxtRobot.ConnectState.CONNECTED)
    {
      tf[2].setText("Bluetooth connection failed");
      return;
    }
    tf[2].setText("Bluetooth OK. Connecting TCP...");
    node = new TcpNode();
    node.addTcpNodeListener(this);
    node.connect(sessionID, nickname);
    while (isRunning)
    {
      int value = rover.getIntensity();
      tf[6].setText("Luminosity value = " + value);
      node.sendMessage("" + value);
      delay(100);
    }
    tf[6].setText("Connection to NXT broken");
  }

  public void onPause()
  {
    if (rover != null)
      rover.exit();
    super.onPause();
  }

  public void notifyConnection(boolean connected)
  {
    if (!connected)
      isRunning = false;
  }

  public void nodeStateChanged(TcpNodeState state)
  {
    if (state == TcpNodeState.CONNECTED)
      tf[3].setText("Connected to relay");
    if (state == TcpNodeState.DISCONNECTED)
      tf[3].setText("Disconnected from relay");
  }

  public void statusReceived(String text)
  {
    L.i("statusReceived: " + text);
    if (text.contains("NxtCommander") && !text.contains("Disconnected"))
    {
      announce(1200);
      tf[5].setText("NxtCommander connected");
      Monitor.wakeUp();
    }
    if (text.contains("Disconnected"))
    {
      announce(800);
      tf[5].setText("NxtCommander disconnected");
      rover.stop();   // For security
    }
  }

  private void announce(int freq)
  {
    playTone(freq, 100);
    delay(600);
    playTone(freq, 100);
  }

  public void messageReceived(String sender, String text)
  {
    playTone(1000, 100);
    switch (text.charAt(0))
    {
      case 'S':
        rover.stop();
        break;
      case 'F':
        rover.forward();
        break;
      case 'B':
        rover.backward();
        break;
      case 'L':
        rover.left();
        break;
      case 'R':
        rover.right();
        break;
    }
  }

}
