// NxtCommander.java

package ch.aplu.nxtcommander;

import ch.aplu.android.*;
import ch.aplu.tcp.*;
import android.graphics.Color;

public class NxtCommander extends GameGrid
  implements GGSoftButtonListener, GGNavigationListener, TcpNodeListener
{
  private final String VERSION = "1.0";
  private Integer vWhite;
  private Integer vBlack;
  private Integer vGreen;
  private static String sessionID = "777";
  private final String myNodeName = "NxtCommander";
  private TcpNode node;
  private NxtActor robot;
  private final int vInterval = 30; // Green color detection interval
  private GGTextField status;

  public NxtCommander()
  {
    setScreenOrientation(PORTRAIT);
    addSoftButton(0, "^");  // Must be done in ctor
    addSoftButton(1, "v");
    addSoftButton(2, "<-");
    addSoftButton(3, "->");
  }

  public void main()
  {
    GGTextField title = new GGTextField("NxtCommander V" + VERSION,
      new Location(0, 20), false);
    title.setTextColor(Color.BLUE);
    status = new GGTextField(new Location(10, getNbVertCells() - 20), false);
    status.setTextColor(Color.BLUE);
    status.show();
    addSoftButtonListener(this);
    addNavigationListener(this);
    robot = new NxtActor();
    setSimulationPeriod(50);
    doRun();
    int w = getNbHorzCells() / 2;
    addActor(robot, new Location(w, w + 30), -90);
    initScreen();
    title.show();
    status.show();
    selectLuminosity();
    connectTcp();
  }

  private void initScreen()
  {
    getBg().clear(WHITE);
    getBg().setPaintColor(Color.BLACK);
    int w = getNbHorzCells() / 2;
    getBg().drawCircle(toPoint(new Location(w, w + 30)), w - 10);
    robot.setLocation(robot.getLocationStart());
    robot.setDirection(robot.getDirectionStart());
    robot.init();
  }

private void selectLuminosity()
{
  GGPreferences prefs = new GGPreferences(this);
  vWhite = prefs.retrieveInt("whiteLevel");
  vBlack = prefs.retrieveInt("blackLevel");
  vGreen = prefs.retrieveInt("greenLevel");

  if (vWhite == null)  // preferences do not exist, set defaults
  {
    vWhite = 600;
    vBlack = 300;
    vGreen = 400;
  }

  vWhite = requestInt("Luminosity Calibration", "White Level", vWhite);
  vBlack = requestInt("Luminosity Calibration", "Black Level", vBlack);
  vGreen = requestInt("Luminosity Calibration", "Green Level", vGreen);

  prefs.storeInt("whiteLevel", vWhite);
  prefs.storeInt("blackLevel", vBlack);
  prefs.storeInt("greenLevel", vGreen);
}

  private void connectTcp()
  {
    node = new TcpNode();
    node.addTcpNodeListener(this);
    status.setText("Connecting '" + node.getRelay() + "'...");
    node.connect(sessionID, myNodeName);
  }

  public void nodeStateChanged(TcpNodeState state)
  {
    if (state == TcpNodeState.CONNECTED)
      status.setText("Connected. Waiting...");
    if (state == TcpNodeState.DISCONNECTED)
      status.setText("Disconnected from relay");
  }

  public void messageReceived(String sender, String text)
  {
    int v = Integer.parseInt(text);
    if (Math.abs(v - vGreen) < vInterval)
      getBg().setPaintColor(Color.rgb(0, 200, 0));
    else
    {
      if (v < vGreen - vInterval)
        getBg().setPaintColor(Color.BLACK);
      else
        getBg().setPaintColor(Color.WHITE);
    }
    getBg().fillCircle(toPoint(robot.getSensorLocation()), 8);
    status.setText("Luminosity reported: " + v);
  }

  public void statusReceived(String text)
  {
    if (text.contains("NxtAgent") && !text.contains("Disconnected"))
      status.setText("NxtAgent connected");
    if (text.contains("Disconnected"))
      status.setText("NxtAgent disconnected.");
  }

  public void buttonPressed(int buttonNb)
  {
    switch (buttonNb)
    {
      case 0:
        node.sendMessage("F");
        robot.forward();
        break;
      case 1:
        node.sendMessage("B");
        robot.backward();
        break;
      case 2:
        node.sendMessage("L");
        robot.left();
        break;
      case 3:
        node.sendMessage("R");
        robot.right();
        break;
    }
  }

  public void buttonReleased(int buttonNb)
  {
  }

  public void buttonClicked(int buttonNb)
  {
  }

  public void buttonRepeated(int buttonNb)
  {
  }

  public void navigationEvent(GGNavigationEvent event)
  {
    switch (event)
    {
      case MENU_DOWN:
        robot.stop();
        node.sendMessage("S");
        showToast("Stop command sent");
        break;
      case BACK_DOWN:
        initScreen();
        showToast("Screen initialized");
        break;
    }
  }

}
