// SensorEvent.java
// Show use of events to get orientation with new sensor API

package ch.aplu.sensor.event;

import android.hardware.*;
import ch.aplu.android.*;
import android.graphics.Typeface;

public class SensorEvent extends GameGrid
  implements GGSensorListener
{
  private float[] R = new float[9];  // Rotation matrix
  private float[] I = new float[9];  // Inclination matrix
  private float[] acc = new float[3];
  private float[] mag = new float[3];
  private float[] orientation = new float[3];
  private final int nbLines = 6;
  private GGTextField[] lines = new GGTextField[nbLines];
  private GGSensor accelerometer;
  private GGSensor magnetometer;

  public SensorEvent()
  {
    super(WHITE);
  }

  public void main()
  {
    setWakeLockEnabled(true);
    initDisplay();

    accelerometer = new GGSensor(this, Sensor.TYPE_ACCELEROMETER,
      SensorManager.SENSOR_DELAY_FASTEST);
    accelerometer.addSensorListener(this);
    accelerometer.setLowPassFilter(10, 5);

    magnetometer = new GGSensor(this, Sensor.TYPE_MAGNETIC_FIELD,
      SensorManager.SENSOR_DELAY_FASTEST);
    magnetometer.addSensorListener(this);
    accelerometer.setLowPassFilter(10, 5);

    while (true)
    {
      synchronized (orientation)
      {
        double azimuth = (360 + Math.toDegrees(orientation[0])) % 360;
        double pitch = Math.toDegrees(orientation[1]);
        double roll = Math.toDegrees(orientation[2]);
        showValue(0, "azimuth", azimuth, "deg");
        e:  showValue(1, "pitch", pitch, "deg");
        showValue(2, "roll", roll, "deg");
        float[] adapted =
          GGSensor.toDeviceRotation(this, orientation, Sensor.TYPE_ORIENTATION);

        double _azimuth = (360 + Math.toDegrees(adapted[0])) % 360;
        double _pitch = Math.toDegrees(adapted[1]);
        double _roll = Math.toDegrees(adapted[2]);
        showValue(3, "*azimuth", _azimuth, "deg");
        showValue(4, "*pitch", _pitch, "deg");
        showValue(5, "*roll", _roll, "deg");
      }
      refresh();
      delay(50);
    }
  }

  public void sensorChanged(float[] values, int sensorType)
  {
    if (sensorType == Sensor.TYPE_ACCELEROMETER)
    {
      for (int i = 0; i < 3; i++)
        acc[i] = values[i];
    }

    if (sensorType == Sensor.TYPE_MAGNETIC_FIELD)
    {
      for (int i = 0; i < 3; i++)
        mag[i] = values[i];
    }
    boolean success =
      SensorManager.getRotationMatrix(R, I, acc, mag);
    if (!success)
      return;

    synchronized (orientation)
    {
      SensorManager.getOrientation(R, orientation);
    }
  }

  private void initDisplay()
  {
    int dist = getNbVertCells() / (nbLines + 1);
    for (int i = 0; i < nbLines; i++)
    {
      lines[i] =
        new GGTextField(new Location(10, (i + 1) * dist), false);
      lines[i].setFontSize(getNbHorzCells() / 15);
      lines[i].setTypeface(Typeface.MONOSPACE);
      lines[i].show();
    }
  }

  private void showValue(int line, String text, double value, String unit)
  {
    lines[line].setText(String.format(text + " = % 5.1f " + unit, value));
  }

}
