// SensorMatrix.java
// Demonstrate use of rotation matrix

package ch.aplu.sensor.matrix;

import android.hardware.*;
import ch.aplu.android.*;
import android.graphics.Typeface;

public class SensorMatrix extends GameGrid
  implements GGSensorListener
{
  private float[] R = new float[9];  // Rotation matrix
  private float[] I = new float[9];  // Inclination matrix
  private float[] acc = new float[3];
  private float[] mag = new float[3];
  private float[] orientation = new float[3];
  private GGConsole c;

  public void main()
  {
    c = GGConsole.init(Typeface.MONOSPACE);
    GGSensor accelerometer = new GGSensor(this, Sensor.TYPE_ACCELEROMETER);
    accelerometer.addSensorListener(this);
    accelerometer.setLowPassFilter(10, 5);

    GGSensor magnetometer = new GGSensor(this, Sensor.TYPE_MAGNETIC_FIELD);
    magnetometer.addSensorListener(this);
    magnetometer.setLowPassFilter(10, 5);
  }

  public void sensorChanged(float[] values, int sensorType)
  {
    c.clear();
    if (sensorType == Sensor.TYPE_ACCELEROMETER)
    {
      for (int i = 0; i < 3; i++)
        acc[i] = values[i];
    }

    if (sensorType == Sensor.TYPE_MAGNETIC_FIELD)
    {
      for (int i = 0; i < 3; i++)
        mag[i] = values[i];
    }
    print("acc_device",
      acc[0], acc[1], acc[2]);
    print("mag_device",
      mag[0], mag[1], mag[2]);

    boolean success =
      SensorManager.getRotationMatrix(R, I, acc, mag);
    c.println("success = " + success);
    if (!success)
      return;

    SensorManager.getOrientation(R, orientation);
    print("azimuth", (360 + Math.toDegrees(orientation[0])) % 360);
    print("pitch", Math.toDegrees(orientation[1]));
    print("roll" , Math.toDegrees(orientation[2]));

    c.println("R = ");
    for (int i = 0; i < 3; i++)
      print("row " + (i+1), R[i], R[3 * i + 1], R[3 * i + 2]);

    float[] g_world = GGSensor.deviceToWorld(R, acc);
    print("g_world", g_world[0], g_world[1], g_world[2]);

    // Transformation test:
    float[] zArrow =
    {
      0, 0, 1
    };
    float[] zArrow_world = GGSensor.deviceToWorld(R, zArrow);
    print("zArrow_world",
      zArrow_world[0], zArrow_world[1], zArrow_world[2]);
    double znorm_world = Math.sqrt(zArrow_world[0] * zArrow_world[0]
      + zArrow_world[1] * zArrow_world[1] + zArrow_world[2] * zArrow_world[2]);
    print("norm", znorm_world);  // assume 1.0
    float[] zArrow_org = GGSensor.worldToDevice(R, zArrow_world);
    print("zArrow_origin",  // assume {0, 0, 1}
      zArrow_org[0], zArrow_org[1], zArrow_org[2]);

    print("Theta", Math.toDegrees(Math.acos(zArrow_world[2])));
  }

  private void print(String f, double... v)
  {
    if (v.length == 1)
      c.println(String.format(f + " = %+4.1f", v[0]));
    else
      c.println(String.format(f + " = %+4.1f, %+4.1f. %+4.1f", 
        v[0], v[1], v[2]));
  }

}
