// ShootingGallery.java
// Calculation of ball cell

package ch.aplu.shootinggallery;

import ch.aplu.android.*;
import android.graphics.Point;
import java.util.ArrayList;

public class ShootingGallery extends GameGrid
  implements GGFlingListener, GGActorCollisionListener
{
  private final int nbBalls = 10;
  private GGStatusBar status;
  private Ball[] balls = new Ball[nbBalls];
  private int width, height;
  private int nbPoints = 0;
  private Figure f;

  public ShootingGallery()
  {
    super(WHITE, false, true, windowZoom(1000));
    setScreenOrientation(LANDSCAPE);
    status = addStatusBar(30);
  }

  public void main()
  {
    width = getNbHorzCells();
    height = getNbVertCells();
    addFlingListener(this);
    setSimulationPeriod(30);
    getBg().setPaintColor(YELLOW);
    getBg().drawLine(0, 2 * height / 10, width, 2 * height / 10);
    getBg().setPaintColor(RED);
    getBg().drawLine(0, height / 2, width, height / 2);
    createBalls();
    status.setText("Fling a ball!");
    doRun();
  }

  public void act()
  {
    if (getNumberOfActors(Ball.class) == 0)  // Game over
    {
      status.setText("Game over. Total points: " + nbPoints);
      f.removeSelf();
      refresh();  // delay will inhibit automatic refresh
      delay(3000);
      createBalls();
      createFigure();
      nbPoints = 0;
      status.setText("Fling a ball!");
    }
    if (getNumberOfActors(Figure.class) == 0)
    {
      delay(1000);
      if (getNumberOfActors(Ball.class) != 0)
        createFigure();
    }
  }

  private void createBalls()
  {
    for (int i = 0; i < nbBalls; i++)
    {
      Ball ball = new Ball(this, (i + 1) * 10);
      int d = width / nbBalls;
      addActorNoRefresh(ball, new Location(d / 2 + i * d, height - d / 2));
      balls[i] = ball;
    }
  }

  private void createFigure()
  {
    f = new Figure();
    double x = 10 + Math.random() * (width - 10);
    addActorNoRefresh(f, new Location(x, height / 10), 
      (Math.random() < 0.5) ? 0 : 180);
    f.setCollisionRectangle(new Point(0, 50), 50, 50);
    for (Ball ball : balls)
    {
      if (ball != null)
      {
        f.addCollisionActor(ball);
        f.addActorCollisionListener(this);
      }
    }
  }

  public boolean flingEvent(Point start, Point end, GGVector velocity)
  {
    Location endLocation = toLocationInGrid(end);
    if (endLocation.y < height / 2)
      return true;
    Location startLocation = toLocationInGrid(start);
    Point startPos = new Point(startLocation.x, startLocation.y);
    int index = getBallIndex(startPos);
    if (index != -1)
    {
      balls[index].launch(endLocation, velocity);
      balls[index] = null; // tag as removed 
    }
    return true;
  }

  // Returns the index of the ball cell where pt resides or
  // -1 if pt is not part of any ball cell
  private int getBallIndex(Point pt)
  {
    int d = width / nbBalls;
    if (pt.y < height - d) // not in lower row
      return -1;
    int index = pt.x / d;
    if (balls[index] == null) // tagged as removed
      return -1;
    return index;
  }

  public int collide(Actor actor1, Actor actor2)
  {
    actor1.removeSelf();
    actor2.hide();
    nbPoints += ((Ball)actor2).getValue();
    displayResult();
    return 10;
  }

  protected void displayResult()
  {
    status.setText("# point: " + nbPoints);
  }

}
